clear all
close all
global solution  % chooses exact solution, see mysolution.m
%
% A FEM MATLAB program. Some crosschecks thrown out.
%
% We load the geometry, exported from a pdetool run.
% The program assumes a setup of pdetool for -Delta u=1, u=1 on boundary
% for checking agains what MATLAB does.
% Done for pderect and pdecirc at 4 scales.
% The whole workspace must be saved
% and is here recovered as R.
% The saves are Rect*.mat and Circ*.mat.
% Assumption: ONLY pw linear FEMs on triangles
%
intext='Circ3S';
R=load (sprintf('%6s.mat',intext));
% rephrasing whatever is there
b=R.b;
p=R.p;
e=R.e;
f=R.f;
c=R.c;
a=R.a;
t=R.t;
% look at points first 
figure
np=length(p(1,:))   % number of points, = trial nodes 
% = centers of pw linear test functions
plot(p(1,:),p(2,:),'rx') % plot them, just to see 
hold on
axis([-1.1 1.1 -1.1 1.1])
axis square
ne=length(e(1,:)) % number of edges, = BOUNDARY edges only!!!!
nt=length(t(1,:)) % number of triangles 
bary=zeros(2,nt); % we save the barycenters of the triangles,
% because they arise as test nodes, where f is sampled.
baryweight=sparse(np,nt); % for comparison, we need the integration matrix of weights
% attached to the barycenters and 
% contributing to the rhs for the equations at each node.
nodeweight=sparse(np,np); % similarly, if we consider the values of f not
% given on barycenters but on nodes. The array assembles the
% weights of the f-values at neighboring nodes.
%
% now loop over triangles and analyze
%
h=0; % this will be the separation distance
%
for j=1:nt
    % these are the indices of the vertices
    i0=t(1,j);
    i1=t(2,j);
    i2=t(3,j);
    % these are the vertices
    p0=[p(1,i0) p(2,i0)]; 
    p1=[p(1,i1) p(2,i1)]; 
    p2=[p(1,i2) p(2,i2)]; 
    % plot the triangle
    plot([p0(1) p1(1) p2(1) p0(1)],[p0(2) p1(2) p2(2) p0(2)]);
    hold on
    % get the barycenter
    bary(:,j)=((p0+p1+p2)/3)';
    % plot it
    plot(bary(1,j),bary(2,j),'x')
    % now the rhs contribution via the triangle area/6
    area=det([1 p0; 1 p1; 1 p2]);
    % radius of circle through vertices:
    h=max(h,norm(p0-p1,2)*norm(p1-p2,2)*norm(p2-p0,2)*0.25/area);
    area6=area/6;
    % and  we store the contribution in our respresentation matrix
    baryweight(i0,j)=area6;
    baryweight(i1,j)=area6;
    baryweight(i2,j)=area6;
    % and the weights for the nodes
    nodeweight(i0,i0)=nodeweight(i0,i0)+area6/3;
    nodeweight(i1,i0)=nodeweight(i1,i0)+area6/3;
    nodeweight(i2,i0)=nodeweight(i2,i0)+area6/3;
    nodeweight(i0,i1)=nodeweight(i0,i1)+area6/3;
    nodeweight(i1,i1)=nodeweight(i1,i1)+area6/3;
    nodeweight(i2,i1)=nodeweight(i2,i1)+area6/3;
    nodeweight(i0,i2)=nodeweight(i0,i2)+area6/3;
    nodeweight(i1,i2)=nodeweight(i1,i2)+area6/3;
    nodeweight(i2,i2)=nodeweight(i2,i2)+area6/3;
end
% now check edges. They turn out to be BOUNDARY only.
% The MATLAB doc does not explicitly say so.
% Also, they sometimes sort the boundary points to be first.
% To play safe, we shall build an index vector bpind for their indices
% We mark this first in
ip=2*ones(np,1);
for i=1:ne
    % mark indices of boundary points 
    ip(e(1,i))=1;
    ip(e(2,i))=1;
    % plot bounday edges
    plot([p(1,e(1,i)) p(1,e(2,i))],[ p(2,e(1,i)) p(2,e(2,i))],'r');
    hold on
end
bpind=find(ip==1); % this gives the list of boundary points
nb=length(bpind);
ppind=find(ip==2); % this is the rest, = PDE points = equations = DOFs


% we need the index of the point at the origin
[val, indzero]=min(sum(p(:,ppind).^2)');

% overplot to see better
plot(p(1,:),p(2,:),'rx')
hold on
plot(p(1,bpind),p(2,bpind),'g+')
%
% we check our f integration matrices
%
figure
spy(baryweight)
title('Composition matrix of rhs in terms of f data in barycenters')
xlabel('Barycentric data points for f')
ylabel('Test nodes = equations')
%
% The following should have the sparsity structure of the stiffness matrix
% because it has nonzero entries exactly for neighbors
%
figure
spy(nodeweight)
title('Composition matrix of rhs in terms of f data in nodes')
xlabel('Nodal data points for f')
ylabel('Test nodes = equations')
% 
% The MATLAB stiffness matrix assembly on the triangulation
%
[StMat, M, FF]=assema(p, t, c, a, 1);
%
% Now for boundary values. The MATLAB way of specifying them seems
% to be procedural, which is useless for our purposes. We need to know
% EXACTLY where the f or g data are sampled. We do NOT use assemb or
% assempde for this reason, but simply plug the Dirichlet boundary
% values into the equations, fixing the values of the boundary nodes
% and using the stiffness matrix only on the rest.
% 
% Here we check how MATLAB solves:
%
% u=assempde(b,p,e,t,c,a,f); % MATLAB solution
% v=zeros(size(u)); % our solution to be
% Dirdat=u(bpind); % extract Dirichlet values 
% FF=FF-StMat(:,bpind)*Dirdat; % correct rhs 
% w=StMat(ppind,ppind)\FF(ppind); % solve for rest via stiffness matrix 
% v(bpind,1)=Dirdat; % and assemble the results 
% v(ppind,1)=w;
% if norm(u-v)>1.0e-10
%     error('Unexpected MATLAB solution technique. Check required.');
% end
%
% The following matrix is the composition matrix for the 
% solution on the non-Dirichlet nodes in terms
% of the Dirichlet boundary data and the f data on barycenters.
%
Cbary=StMat(ppind,ppind)\[baryweight(ppind,:) -StMat(ppind,bpind) ];
%
% The following matrix is the composition matrix for the 
% solution on the non-Dirichlet nodes in terms
% of nb Dirichlet boundary data and the f data on nodes.
%
Cnode=StMat(ppind,ppind)\[nodeweight(ppind,:) -StMat(ppind,bpind)];
%
% Now we can theck this finally for cases we know
%
solution='parabola';
[val, lapval]=mysolution(p(1,:)',p(2,:)');
Dirdat=val(bpind,1);
PDEdat=-lapval;
uc(bpind,1)=Dirdat; 
sD=size(Dirdat)
sP=size(PDEdat)
sC=size(Cnode)
uc(ppind,1)=Cnode*[PDEdat; Dirdat]; % 
%
figure
pdeplot(p,e,t,'xydata',uc,'zdata',uc,'mesh','off');
title('Solution for f data on nodes');
%
figure
pdeplot(p,e,t,'xydata',val-uc,'zdata',val-uc,'mesh','on');
title('Error for f data on nodes');
%
% now another solution on barycenters
%
solution='sinsin';
[valp, lapvalp]=mysolution(p(1,:)',p(2,:)');
[valb, lapvalb]=mysolution(bary(1,:)',bary(2,:)');

Dirdat=valp(bpind);
PDEdat=zeros(nt,1);
PDEdat=-lapvalb;
uc(bpind,1)=Dirdat; 
uc(ppind,1)=Cbary*[PDEdat; Dirdat]; % 

figure
pdeplot(p,e,t,'xydata',uc,'zdata',uc,'mesh','off');
title('Solution for f data on barycenters');

figure
pdeplot(p,e,t,'xydata',valp-uc,'zdata',valp-uc,'mesh','on');
title('Error for f data on barycenters');
disp(sprintf('%6s_out',intext))
save(sprintf('%6s_out',intext))
%
% Here are the data for the table in the paper
%
[ne nt np h]