clear all
close all
%
% this is to plot a FEM power function 
%
% first get all FEM data on calculation grid. This is a variation of
% FEMEvaluator.m
%
intext='Circ2S';
disp(sprintf('%6s_out.mat',intext))
R=load(sprintf('%6s_out.mat',intext))
dirpoints=[R.p(1,R.bpind)' R.p(2,R.bpind)'];
ndirpoints=length(dirpoints(:,1));
lappoints=[R.p(1,R.ppind)' R.p(2,R.ppind)'];
nlappoints=length(lappoints(:,1));
barypoints=[R.bary(1,:)' R.bary(2,:)'];
nbarypoints=length(barypoints(:,1));
trialpoints=[R.p(1,:)' R.p(2,:)'];
ntrialpoints=length(trialpoints(:,1))
Cnode=R.Cnode;
Cbary=R.Cbary;
sCnode=size(Cnode)
sCbary=size(Cbary)
plot(dirpoints(:,1),dirpoints(:,2),'x')
hold on
plot(trialpoints(:,1),trialpoints(:,2),'o')
axis square
legend('Dirichlet','nodes',-1)
nt=length(R.t(1,:));
t=R.t;
p=R.p;
% We need the midpoints of ALL edges. 
edgelist=[];
%  loop over triangles, see FEM01.m
for j=1:nt
    % these are the indices of the vertices
    i0=t(1,j);
    i1=t(2,j);
    if i0<i1
        edgelist=[edgelist; [i0 i1]];
    else
        edgelist=[edgelist; [i1 i0]];
    end
    i2=t(3,j);
    if i0<i2
        edgelist=[edgelist; [i0 i2]];
    else
        edgelist=[edgelist; [i2 i0]];
    end
    if i1<i2
        edgelist=[edgelist; [i1 i2]];
    else
        edgelist=[edgelist; [i2 i1]];
    end
    % these are the vertices
    p0=[p(1,i0) p(2,i0)]; 
    p1=[p(1,i1) p(2,i1)]; 
    p2=[p(1,i2) p(2,i2)]; 
    % plot the triangle
    plot([p0(1) p1(1) p2(1) p0(1)],[p0(2) p1(2) p2(2) p0(2)]);
    hold on
end
% edgelist still has duplicates 
ne=length(edgelist);
[s si]=sort(edgelist(:,2)+edgelist(:,1)*ne);
edgelist=edgelist(si,:);
elist=[edgelist(1,:)]; % this will be the edgelist without duplicates
k=1;
for i=2:length(edgelist(:,1))
    if edgelist(i,1)>elist(k,1)
        k=k+1;
        elist=[elist; [edgelist(i,:)]];
        continue;
    end
    if edgelist(i,1)==elist(k,1)
        if edgelist(i,2)>elist(k,2)
            k=k+1;
            elist=[elist; [edgelist(i,:)]];
            continue;
        end
    end   
end
le=length(elist(:,1));
% now for the midpoints mp we need
mp=zeros(le,2);
for j=1:le
    p0=[p(1,elist(j,1)) p(2,elist(j,1))];
    p1=[p(1,elist(j,2)) p(2,elist(j,2))];
    mp(j,:)=(p0+p1)/2;
    plot(mp(j,1), mp(j,2),'rx')
    hold on
    plot(p0(1,1),p0(1,2),'bx')
    hold on
    plot(p1(1,1),p1(1,2),'g+')
    hold on   
end
% [mp elist]
% 
% We need prediction data on the midpoints mp of all edges.
% There are 3 kinds: with 0, 1, or 2 neighbors on the boundary
% using ppind and bpind, see FEM01.m
%
ppind=R.ppind;
bpind=R.bpind;
Cnew=zeros(le,ntrialpoints+ndirpoints);
for j=1:le
    i0=elist(j,1);
    i1=elist(j,2);
    b0=length(find(bpind==i0));
    b1=length(find(bpind==i1));
    if b0+b1==2
        plot(mp(j,1),mp(j,2),'bo');
        hold on
        % this s a node between two boundary points
        ip0=find(bpind==i0);
        ip1=find(bpind==i1);
        Cnew(j,ntrialpoints+ip0)=0.5;
        Cnew(j,ntrialpoints+ip1)=0.5;
    end
    if b0+b1==1
        plot(mp(j,1),mp(j,2),'go');
        hold on
        % this is a node neighboring a single boundary node
        if b0==1
            ip0=find(bpind==i0);
            ip1=find(ppind==i1);
            Cnew(j,:)=Cnode(ip1,:)/2;
            Cnew(j,ntrialpoints+ip0)=Cnew(j,ntrialpoints+ip0)+0.5;
        else
            ip0=find(ppind==i0);
            ip1=find(bpind==i1);
            Cnew(j,:)=Cnode(ip0,:)/2;
            Cnew(j,ntrialpoints+ip1)=Cnew(j,ntrialpoints+ip1)+0.5;
        end
    end
    if b0+b1==0
        plot(mp(j,1),mp(j,2),'mo');
        hold on
        % this is a point between two PDE points
        ip0=find(ppind==i0);
        ip1=find(ppind==i1);
        Cnew(j,:)=(Cnode(ip0,:)+Cnode(ip1,:))/2;
    end  
end
% figure
% spy(Cnew)
% figure
% spy(Cnode)
%
% Fix Sobolev order here
%
SobOrder=5
SobScale=1
%
% now get norms on all midpoints. Very primitively programmed, because the
% kernel matrices are recalculated plenty of times.
%
PFe=zeros(le,1);
for k=1:le
    Cvector=Cnew(k,:)';
    PFe(k,1)=ErrorNormOfVector(Cvector, mp(k,:), trialpoints, dirpoints,  SobOrder, SobScale);
    % [PFe(k,1) mp(k,:)]
end
PFn=zeros(nlappoints,1);
for k=1:nlappoints
    Cvector=Cnode(k,:)';
    PFn(k,1)=ErrorNormOfVector(Cvector, lappoints(k,:), trialpoints, dirpoints,  SobOrder, SobScale);
end
%
% to plot, we need the data for the finer triangulation
% and find the correspondence to the nodes there.
%
intext2='Circ3S';
disp(sprintf('%6s_out.mat',intext2));
R2=load(sprintf('%6s_out.mat',intext2))
dirpoints2=[R2.p(1,R2.bpind)' R2.p(2,R2.bpind)'];
ndirpoints2=length(dirpoints2(:,1));
lappoints2=[R2.p(1,R2.ppind)' R2.p(2,R2.ppind)'];
nlappoints2=length(lappoints2(:,1));
barypoints2=[R2.bary(1,:)' R2.bary(2,:)'];
nbarypoints2=length(barypoints2(:,1));
trialpoints2=[R2.p(1,:)' R2.p(2,:)'];
ntrialpoints2=length(trialpoints2(:,1))
%
% We loop through all trialpoints2 to get the right values
%
PFull=zeros(ntrialpoints2,1);
figure
for i=1:ntrialpoints2
    pt=trialpoints2(i,:);
    found=0;
    % check if midpoint
    for k=1:le
        if norm(pt-mp(k,:),Inf)<0.05
            % disp('midpoint')
            PFull(i,1)=PFe(k,1);
            % wert= PFull(i,1)
            found=1;
            break;
        end  
    end
    % check if old boundary point
    for k=1:ndirpoints
        if norm(pt-dirpoints(k,:),Inf)<0.05
            % disp('old dirpoint')
            PFull(i,1)=0;
            found=1;
            break;
        end 
    end
    % check if old PDE node point
    for k=1:nlappoints
        if norm(pt-lappoints(k,:),Inf)<0.05
            % disp('Old lappoint')
            PFull(i,1)=PFn(k,1);
            % wert=PFull(i,1)
            found=1;
            break;
        end    
    end 
    if found==0
        disp('Point not found')
        i
        trialpoints2(i,:)
        norm(trialpoints2(i,:),2)
        plot(trialpoints2(i,1),trialpoints2(i,2),'k.')
        hold on
    end
end
hold on
plot(mp(:,1),mp(:,2),'bo')
hold on
plot(lappoints(:,1),lappoints(:,2),'r+')
hold on
plot(dirpoints(:,1),dirpoints(:,2),'gx')

% PFull

figure
pdeplot(R2.p,R2.e,R2.t,'xydata',PFull,'zdata',PFull,'mesh','on');
title('Error on refined nodes');

figure
pdecont(R2.p,R2.t,PFull);
title('Error on refined nodes');
hold on
plot(trialpoints2(:,1),trialpoints2(:,2),'kx')
axis square
