clear all;
close all;
global RBFtype
global RBFscale
global RBFpar
RBFtype='ms'
RBFpar=3  % Sobolev order will be one larger, we are in 2D
RBFscale=1;
h=0.05 % the h for the points to be offered
%
% generate points to be offered, boundary first on circle
%
nb=floor(2*pi/h); 
hb=2*pi/nb;
bndpoints=[cos(0:h:2*pi-h)' sin(0:h:2*pi-h)'];
figure
plot(bndpoints(:,1),bndpoints(:,2),'x');
axis square
% now for the interior
[xp, yp]=meshgrid(-1:h:1,-1:h:1);
xxp=xp(:);
yyp=yp(:);
[ff fi]=find(xxp.^2+yyp.^2<=(1-h/4)^2); % picking truly interior points
intlappoints=[xxp(ff,1) yyp(ff,1)];
nintlappoints=length(ff);
hold on
plot(intlappoints(:,1),intlappoints(:,2),'.')
% we add the boundary points to the Laplace points for Laplace collocation
lappoints=[intlappoints; bndpoints];
nlappoints=length(lappoints(:,1))
figure
plot(lappoints(:,1),lappoints(:,2),'.')
% now get close to the origin for a starting point
[am ai]=min(lappoints(:,1).^2+lappoints(:,2).^2);
startpoint=lappoints(ai,:);
%
restpointlist=[(1:nlappoints)]; % holds the lappoint indices that are still eligible
bndpointlist=[1];   % holds the indices of used boundary points
lappointlist=[ai];  % holds the indices of used Laplacian points
ipi=find(restpointlist~=ai);
restpointlist=restpointlist(ipi); % this must be the complement of the lappointlist
nlp=1; % counter for used lappoints
nbp=1; % counter for used boundary points
%
% If the problem is small, we can precalculate ALL kernal values.
% Otherwise, skip this and recalculate the matrices on the fly.
%
ALL=laplaceXYkermat(lappoints,lappoints);
ALB=laplacekermat(lappoints,bndpoints);
AL=laplacekermat(lappoints,lappoints);
ABB=kermat(bndpoints,bndpoints);
AB=kermat(bndpoints,lappoints);
% A=[ALL ALB; ALB' ABB];
errlist=[]; % we want to monitor the gain
for j=1:120 % fix the max number of steps here
    % get the local kernel matrix, either by copy or calculation
    Aloc=[ALL(lappointlist,lappointlist) ALB(lappointlist, bndpointlist);...
            ALB(lappointlist,bndpointlist)' ABB(bndpointlist,bndpointlist)];
    % rhs, first on boundary points
    rhslocbnd=[ALB(lappointlist,:); ABB(bndpointlist,:)];
    ca=condest(Aloc);
    if ca> 1.0e14
        disp(sprintf('Stopped due to large local condition %e',ca))
        break;
    end
    cobnd=Aloc\rhslocbnd; % coefficients for the boundary
    pfbnd=frbf(0,0)-sum(cobnd.*(rhslocbnd)); % error norm squared on ALL  boundary points
    btst=norm(pfbnd(bndpointlist),Inf) % just a check if we have zeros on the used boundary points
    [mpbnd ipbnd]= max(pfbnd) % we take the maximum on the boundary
    % Now on PDE points, different rhs 
    rhsloclap=[AL(lappointlist,:); AB(bndpointlist,:) ];
    colap=Aloc\rhsloclap; % coefficients
    pflap=frbf(0,0)-sum(colap.*(rhsloclap)); % error norm squared on the lappoints
    % plotting each 10-th case
    if mod(j,10)==0
        figure
        subplot(1,3,1)
        plot(bndpoints(bndpointlist,1),bndpoints(bndpointlist,2),'x',...
            lappoints(lappointlist,1),lappoints(lappointlist,2),'.')
        axis square
        subplot(1,3,2)
        plot(1:length(pfbnd),sqrt(abs(pfbnd)))
        axis square
        val=zeros(length(xxp),1);
        val(ff,1)=pflap(1,1:nintlappoints)';       
        subplot(1,3,3);
        surf(xp,yp,reshape(sqrt(val),size(xp)))
        axis square
        % title(sprintf('After %d steps of the greedy method',j))
    end
    % now get maximum of norm on Laplacian rest points.
    [mplap iplap]= max(pflap(restpointlist))
    iplap=restpointlist(iplap);
    % if larger on boundary, use boundary point
    if mpbnd>=mplap
        nbp=nbp+1;
        bndpointlist=[bndpointlist ipbnd];
    else
        nlp=nlp+1;
        ipi=find(restpointlist~=iplap);
        restpointlist=restpointlist(ipi);
        lappointlist=[lappointlist iplap];
    end
    errlist=[errlist max(mpbnd, mplap)]; % monitor the error
end
figure
semilogy(sqrt(errlist))
title('Maximum error norm as function of number of greedily selected points')
figure
loglog(sqrt(1:length(errlist)),sqrt(errlist))
title('Maximum error norm as function of sqrt(n)')
figure
subplot(1,3,1)
plot(bndpoints(bndpointlist,1),bndpoints(bndpointlist,2),'x',...
    lappoints(lappointlist,1),lappoints(lappointlist,2),'.')
axis square
subplot(1,3,2)
plot(1:length(pfbnd),sqrt(abs(pfbnd)))
axis square
val=zeros(length(xxp),1);
%
val(ff,1)=pflap(1,1:nintlappoints)';
%
subplot(1,3,3);
surf(xp,yp,reshape(sqrt(val),size(xp)))
axis square
disp(sprintf('After %d steps of the greedy method',j))
